@@ -3,6 +3,7 @@ from __future__ import with_statement
 import os
 import re
 import tempfile
+import time
 import shutil
 
 from datetime import datetime
@@ -12,15 +13,57 @@ from fabric.state import env
 from fabric.operations import require
 from fabric.contrib.console import confirm
 
-from .base import get_local_base_dir, backup_create_local
-
-
 __all__ = ("update_env", "download_uploads_from_remote",
            "upload_uploads_to_remote", "download_database_from_remote",
            "update_database_from_remote", "update_database_on_remote",
            "deploy")
 
 
+def get_local_base_dir():
+    return os.path.dirname(os.path.abspath(env.real_fabfile))
+
+
+def backup_prune():
+    if not env.backup_path:
+        return
+
+    backup_path = os.path.join(env.local_base_dir, env.backup_path)
+
+    if not os.path.exists(backup_path):
+        return
+
+    wanted = []
+
+    for entry in os.listdir(backup_path):
+        if entry.startswith("."):
+            continue
+
+        absentry = os.path.join(backup_path, entry)
+
+        if time.time() - os.path.getctime(absentry) > 43200:
+            wanted.append(absentry)
+
+    for entry in wanted:
+        shutil.rmtree(entry)
+
+
+def backup_create_local(local_source):
+    backup_prune()
+
+    backup_target = os.path.join(env.local_base_dir,
+                                 env.backup_path,
+                                 env.backup_prefix)
+
+    if os.path.isdir(local_source):
+        backup_target = os.path.join(backup_target, local_source)
+
+    local("mkdir -p %s" % backup_target)
+
+    local("rsync -a --link-dest=%s %s %s" % (
+        local_source, local_source, backup_target)
+    )
+
+
 def update_env(*args, **kwargs):
     env.wordpress_path = "public_html"
     env.backup_prefix = "%Y%m%d-%H%m%S"
@@ -84,7 +127,7 @@ def download_database_from_remote():
 
     wp_config = read_config()
 
-    run("mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > /tmp/%(DB_USER)s.sql" % wp_config['remote'])
+    run("mysqldump -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > /tmp/%(DB_USER)s.sql" % wp_config['remote'])
     get("/tmp/%(DB_USER)s.sql" % wp_config['remote'], os.path.join(env.local_base_path, env.local_wordpress_path, "%(DB_USER)s.sql" % wp_config['remote']))
     run("rm -f /tmp/%(DB_USER)s.sql" % wp_config['remote'])
 
@@ -100,17 +143,17 @@ def update_database_from_remote():
     wp_config['local']['dump_filename'] = dump_filename
 
     with lcd(env.local_base_dir):
-        local("mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s | gzip > %(DB_NAME)s.sql.gz" % wp_config['local'])
+        local("mysqldump -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s --add-drop-table %(DB_NAME)s | gzip > %(DB_NAME)s.sql.gz" % wp_config['local'])
 
         backup_create_local("%(DB_NAME)s.sql.gz" % wp_config['local'])
 
         run("rm -f %(DB_NAME)s.sql.gz" % wp_config['local'])
 
-    run("mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > %(dump_filename)s" % wp_config['remote'])
+    run("mysqldump -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > %(dump_filename)s" % wp_config['remote'])
     get(dump_filename, dump_filename)
     run("rm -f %s" % dump_filename)
 
-    local("mysql -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s %(DB_NAME)s < %(dump_filename)s" % wp_config['local'])
+    local("mysql -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s %(DB_NAME)s < %(dump_filename)s" % wp_config['local'])
 
     os.unlink(dump_filename)
 
@@ -123,15 +166,15 @@ def update_database_on_remote():
 
     wp_config = read_config()
 
-    local("mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > /tmp/%(DB_USER)s.sql" % wp_config['local'])
+    local("mysqldump -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s --add-drop-table %(DB_NAME)s > /tmp/%(DB_USER)s.sql" % wp_config['local'])
     put("/tmp/%(DB_USER)s.sql" % wp_config['local'], "/tmp/%(DB_USER)s.sql" % wp_config['remote'])
     os.unlink("/tmp/%(DB_USER)s.sql" % wp_config['local'])
 
     wp_config['remote']['backup_name'] = "%(DB_USER)s_%(DB_NAME)s" % wp_config['remote']
 
-    run("mysqldump -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s --add-drop-table %(DB_NAME)s | gzip > /tmp/%(backup_name)s.sql.gz" % wp_config['remote'])
+    run("mysqldump -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s --add-drop-table %(DB_NAME)s | gzip > /tmp/%(backup_name)s.sql.gz" % wp_config['remote'])
 
-    run("mysql -u%(DB_USER)s -p%(DB_PASSWORD)s -h%(DB_HOST)s %(DB_NAME)s < /tmp/%(DB_USER)s.sql" % wp_config['remote'])
+    run("mysql -u%(DB_USER)s -p\"%(DB_PASSWORD)s\" -h%(DB_HOST)s %(DB_NAME)s < /tmp/%(DB_USER)s.sql" % wp_config['remote'])
     run("rm -f /tmp/%(DB_USER)s.sql" % wp_config['remote'])
 
 
